;############################################################
; file: plotOpensees.lsp
; author: Hanlin DONG (http://www.hanlindong.com)
; version: 1.1
; license: MIT License (https://opensource.org/licenses/MIT)
; last update: 2017-05-07 12:00
; readme: 
;   This file plots the opensees model printed by plotOpensees.tcl
;   You can find plotOpensees.tcl at www.hanlindong.com
;############################################################

(defun c:plotOpensees(/ filename) 
    ; create layers
    (entmake (list'(0 . "LAYER")'(100 . "AcDbSymbolTableRecord")'(100 . "AcDbLayerTableRecord") '(62 . 1) '(370 . 0) '(70 . 0) '(290 . 1) '(2 . "nodes")))
    (entmake (list'(0 . "LAYER")'(100 . "AcDbSymbolTableRecord")'(100 . "AcDbLayerTableRecord") '(62 . 2) '(370 . 0) '(70 . 0) '(290 . 1) '(2 . "elements")))
    (entmake (list'(0 . "LAYER")'(100 . "AcDbSymbolTableRecord")'(100 . "AcDbLayerTableRecord") '(62 . 3) '(370 . 0) '(70 . 0) '(290 . 1) '(2 . "nodeTag")))
    (entmake (list'(0 . "LAYER")'(100 . "AcDbSymbolTableRecord")'(100 . "AcDbLayerTableRecord") '(62 . 4) '(370 . 0) '(70 . 0) '(290 . 1) '(2 . "eleTag")))
    ; choose text file
    (setq filename (getfiled "select file" "/" "txt" 0))
    ; set text height
    (setq textsize (getint "text height<200>:"))
    (if (= textsize nil) (setq textsize 200))
    ; read file
    (setq f (open filename "r"))
    (setq line (read-line f))
    (setq line (read-line f))
    ; table of node coordinates
    (setq nodecoords ())
    ; read node and create table
    (while (= line "node") (progn
        ; read node tag
        (setq line (read-line f))
        (setq nodetag line)
        ; read coordinates
        (setq line (read-line f))
        (setq xcoord (atof line))
        (setq line (read-line f))
        (setq ycoord (atof line))
        (setq line (read-line f))
        (setq zcoord (atof line))
        ; push in the coordinates table
        (setq point (list xcoord ycoord zcoord))
        (setq node (list nodetag point))
        (setq nodecoords (cons node nodecoords))
        ; draw node and nodetag
        (entmake (list '(0 . "POINT") '(8 . "nodes") (cons 10 point)))
        (entmake (list '(0 . "TEXT") '(8 . "nodeTag") (cons 10 point) (cons 1 nodetag) (cons 40 textsize)))
        ; read "node"
        (setq line (read-line f))
    ))
    (setq line (read-line f))
    (while (= line "ele") (progn
        ; read ele tag
        (setq line (read-line f))
        (setq eletag line)
        ; read iNode and jNode
        (setq line (read-line f))
        (setq iNode line)
        (setq line (read-line f))
        (setq jNode line)
        ; draw line
        (setq iNodeCoord (cadr (assoc iNode nodecoords)))
        (setq jNodeCoord (cadr (assoc jNode nodecoords)))
        (entmake (list '(0 . "LINE") '(8 . "elements") (cons 10 iNodeCoord) (cons 11 jNodeCoord)))
        ; find mid point of element
        (setq midx (/ (+ (car iNodeCoord) (car jNodeCoord)) 2))
        (setq midy (/ (+ (cadr iNodeCoord) (cadr jNodeCoord)) 2))
        (setq midz (/ (+ (caddr iNodeCoord) (caddr jNodeCoord)) 2))
        (setq midpoint (list midx midy midz))
        (entmake (list '(0 . "TEXT") '(8 . "eleTag") (cons 10 midpoint) (cons 1 eletag) (cons 40 textsize)))
        ; read "ele"
        (setq line (read-line f))
    ))
    (close f)
)